#include "stdafx.h"
#include <atomic>
#include "IMSSystem.h"
//#include "hwAOD_ISOMET.h"

class ImageDownloadSupervisor : public iMS::IEventHandler
{
private:
	std::atomic<bool> m_downloading{ true };
	std::atomic<bool> m_error{ false };
public:
	void EventAction(void* sender, const int message, const int param)
	{
		switch (message)
		{
		case (iMS::ImageDownloadEvents::DOWNLOAD_FINISHED): 
			//TRACE0("Image Download Finished!\n"); 
			m_downloading.store(false); 
			break;
		case (iMS::ImageDownloadEvents::DOWNLOAD_ERROR):
			m_error.store(true); 
			break;
		}
	}
	bool Busy() const { return m_downloading.load(); }
	void Reset() { m_downloading.store(true); m_error.store(false); }
	bool Error() const { return m_error.load(); };
};

class ImageVerifySupervisor : public iMS::IEventHandler
{
private:
	std::atomic<bool> m_verifying{ true };
public:
	void EventAction(void* sender, const int message, const int param)
	{
		switch (message)
		{
		case (iMS::ImageDownloadEvents::VERIFY_SUCCESS): 
			//std::cout << "Image Verify Successful!" << std::endl; 
			m_verifying.store(false); 
			break;
		case (iMS::ImageDownloadEvents::VERIFY_FAIL): 
			//std::cout << "Image Verify FAILED! First error byte: " << param << std::endl; 
			m_verifying.store(false); 
			break;
		}
	}
	bool Busy() const { return m_verifying.load(); }
	void Reset() { m_verifying.store(true); }
};

class CompensationTableDownloadSupervisor : public iMS::IEventHandler
{
private:
	std::atomic<bool> m_downloading{ true };
	std::atomic<bool> m_error{ false };
public:
	void EventAction(void* sender, const int message, const int param)
	{
		switch (message)
		{
		case (iMS::CompensationEvents::DOWNLOAD_FINISHED): 
			//TRACE0("Comp Table Download Finished!\n"); 
			m_downloading.store(false); 
			break;
		case (iMS::CompensationEvents::DOWNLOAD_ERROR): 
			m_error.store(true);  
			break;
		}
	}
	bool Busy() const { return m_downloading.load(); };
	bool Error() const { return m_error.load(); };
};

class CompensationTableVerifySupervisor : public iMS::IEventHandler
{
private:
	std::atomic<bool> m_verify_success{ false };
	std::atomic<bool> m_verifying{ true };
public:
	void EventAction(void* sender, const int message, const int param)
	{
		switch (message)
		{
		case (iMS::CompensationEvents::VERIFY_SUCCESS): 
			//TRACE0("Comp Table Verify Successful!\n"); 
			m_verify_success.store(true); 
			m_verifying.store(false); 
			break;
		case (iMS::CompensationEvents::VERIFY_FAIL): 
			//TRACE0("Comp Table Verify FAILED!\n"); 
			m_verifying.store(false); 
			break;
		}
	}
	bool Busy() const { return m_verifying.load(); };
	bool Success() const { return m_verify_success.load(); };
};

class ToneBufferDownloadSupervisor : public iMS::IEventHandler
{
private:
	std::atomic<bool> m_downloading{ true };
	std::atomic<bool> m_error{ false };
public:
	void EventAction(void* sender, const int message, const int param)
	{
		switch (message)
		{
		case (iMS::ToneBufferEvents::DOWNLOAD_FINISHED): 
			//std::cout << "Tone Buffer Download Finished!" << std::endl; 
			m_downloading.store(false); 
			break;
		case (iMS::ToneBufferEvents::DOWNLOAD_ERROR): 
			m_error.store(true); 
			break;
		}
	}
	bool Busy() const { return m_downloading.load(); };
	bool Error() const { return m_error.load(); };
};

class PlaybackSupervisor : public iMS::IEventHandler
{
private:
	std::atomic<bool> m_playing{ false };
public:
	void EventAction(void* sender, const int message, const int param)
	{
		switch (message)
		{
		case (iMS::ImagePlayerEvents::IMAGE_STARTED): 
			//std::cout << "Image Playback Started" << std::endl; 
			m_playing.store(true); 
			break;
		case (iMS::ImagePlayerEvents::POINT_PROGRESS): 
			//std::cout << "Progress: " << param << std::endl; 
			break;
		case (iMS::ImagePlayerEvents::IMAGE_FINISHED): //std::cout << "Image Playback Finished" << std::endl; 
			m_playing.store(false); 
			break;
		}
	}
	bool Busy() const { return m_playing.load(); };
};

class SystemFuncSupervisor : public iMS::IEventHandler
{
private:
	std::atomic<bool> m_downloading{ true };
	std::atomic<bool> m_error{ false };
public:
	void EventAction(void* sender, const int message, const double param)
	{
		switch (message)
		{
		case (iMS::SystemFuncEvents::SYNTH_TEMPERATURE_1) : 
			//TRACE1("System Temperature #1: %f C\n", param); 
			if(param > 0.0 && param < 100.0)	dTemp1 = param;
			break;
		case (iMS::SystemFuncEvents::SYNTH_TEMPERATURE_2) : 
			//TRACE1("System Temperature #2: %f C\n", param);
			if (param > 0.0 && param < 100.0)	dTemp2 = param;
			break;
		}
	}
	bool Busy() const { return m_downloading.load(); }
	void Reset() { m_downloading.store(true); m_error.store(false); }
	bool Error() const { return m_error.load(); };
	double dTemp1;
	double dTemp2;
};

